@@ -219,12 +219,13 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 	pool_signal(SIGPIPE, SIG_IGN);
 
 	/* create unix domain socket */
-	fds = malloc(sizeof(int));
+	fds = malloc(sizeof(int) * 2);
 	if (fds == NULL)
 		ereport(FATAL,
 				(errmsg("failed to allocate memory in startup process")));
 
 	fds[0] = create_unix_domain_socket(un_addr);
+	fds[1] = -1;
 	on_proc_exit(FileUnlink, (Datum) un_addr.sun_path);
 
 	/* create inet domain socket if any */
@@ -310,8 +311,10 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 	/* Add onproc exit to clean up the unix domain socket at exit */
 	on_proc_exit(FileUnlink, (Datum)pcp_un_addr.sun_path);
 
-    /* maybe change "*" to pool_config->pcp_listen_addresses */
-	pcp_inet_fd = create_inet_domain_socket("*", pool_config->pcp_port);
+	if (pool_config->pcp_listen_addresses[0])
+	{
+		pcp_inet_fd = create_inet_domain_socket(pool_config->pcp_listen_addresses, pool_config->pcp_port);
+	}
 	pcp_pid = pcp_fork_a_child(pcp_unix_fd, pcp_inet_fd, pcp_conf_file);
 
 	/* Fork worker process */
@@ -1015,9 +1018,11 @@ static void terminate_all_childrens()
     /* wait for all children to exit */
     do
     {
-        wpid = wait(NULL);
-    }while (wpid > 0 || (wpid == -1 && errno == EINTR));
-    
+		int ret_pid;
+
+        wpid = waitpid(-1, &ret_pid, WNOHANG);
+    } while (wpid > 0 || (wpid == -1 && errno == EINTR));
+
     if (wpid == -1 && errno != ECHILD)
         ereport(LOG,
                 (errmsg("wait() failed. reason:%s", strerror(errno))));
@@ -1208,6 +1213,8 @@ static RETSIGTYPE exit_handler(int sig)
 {
 	int i;
     pid_t wpid;
+
+	int save_errno = errno;
 	POOL_SETMASK(&AuthBlockSig);
 
 	/*
@@ -1239,6 +1246,7 @@ static RETSIGTYPE exit_handler(int sig)
 				 errdetail("ignoring...")));
 
 		POOL_SETMASK(&UnBlockSig);
+		errno = save_errno;
 		return;
 	}
 	exiting = 1;
@@ -1278,15 +1286,19 @@ static RETSIGTYPE exit_handler(int sig)
 	POOL_SETMASK(&UnBlockSig);
     do
     {
-        wpid = wait(NULL);
-    }while (wpid > 0 || (wpid == -1 && errno == EINTR));
-    
+		int ret_pid;
+
+        wpid = waitpid(-1, &ret_pid, WNOHANG);
+    } while (wpid > 0 || (wpid == -1 && errno == EINTR));
+
     if (wpid == -1 && errno != ECHILD)
         ereport(LOG,
                 (errmsg("wait() failed. reason:%s", strerror(errno))));
 
 	process_info = NULL;
 	exit(0);
+
+	errno = save_errno;
 }
 
 /*
@@ -1329,6 +1341,8 @@ static int get_next_master_node(void)
  */
 static RETSIGTYPE failover_handler(int sig)
 {
+	int save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	failover_request = 1;
 	if(write(pipe_fds[1], "\0", 1) < 0)
@@ -1336,6 +1350,8 @@ static RETSIGTYPE failover_handler(int sig)
                 (errmsg("failover_handler: write to pipe failed with error \"%s\"", strerror(errno))));
 
 	POOL_SETMASK(&UnBlockSig);
+
+	errno = save_errno;
 }
 
 /*
@@ -1904,9 +1920,11 @@ static void failover(void)
  */
 static RETSIGTYPE health_check_timer_handler(int sig)
 {
+	int save_errno = errno;
 	POOL_SETMASK(&BlockSig);
 	health_check_timer_expired = 1;
 	POOL_SETMASK(&UnBlockSig);
+	errno = save_errno;
 }
 
 
@@ -1987,6 +2005,8 @@ do_health_check(bool use_template_db, volatile int *health_check_node_id)
  */
 static RETSIGTYPE reap_handler(int sig)
 {
+	int save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	sigchld_request = 1;
 	if(write(pipe_fds[1], "\0", 1) < 0)
@@ -1994,6 +2014,8 @@ static RETSIGTYPE reap_handler(int sig)
             (errmsg("reap_handler: write to pipe failed with error \"%s\"", strerror(errno))));
 
 	POOL_SETMASK(&UnBlockSig);
+
+	errno = save_errno;
 }
 
 /*
@@ -2241,12 +2263,16 @@ static void wakeup_children(void)
 
 static RETSIGTYPE wakeup_handler(int sig)
 {
+	int save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	wakeup_request = 1;
 	if(write(pipe_fds[1], "\0", 1) < 0)
         ereport(WARNING,
             (errmsg("wakeup_handler: write to pipe failed with error \"%s\"", strerror(errno))));
 	POOL_SETMASK(&UnBlockSig);
+
+	errno = save_errno;
 }
 
 /*
@@ -2255,6 +2281,8 @@ static RETSIGTYPE wakeup_handler(int sig)
  */
 static RETSIGTYPE reload_config_handler(int sig)
 {
+	int save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	reload_config_request = 1;
 	if(write(pipe_fds[1], "\0", 1) < 0)
@@ -2262,6 +2290,8 @@ static RETSIGTYPE reload_config_handler(int sig)
             (errmsg("reload_config_handler: write to pipe failed with error \"%s\"", strerror(errno))));
 
 	POOL_SETMASK(&UnBlockSig);
+
+	errno = save_errno;
 }
 
 static void kill_all_children(int sig)
