@@ -352,6 +352,12 @@ void do_child(int *fds)
 		 */
 		pool_init_session_context(child_frontend, backend);
 
+		/*
+		 * Set protocol versions
+		 */
+		pool_set_major_version(sp->major);
+		pool_set_minor_version(sp->minor);
+
 		/*
 		 * Mark this connection pool is connected from frontend 
 		 */
@@ -920,6 +926,8 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
  */
 static RETSIGTYPE die(int sig)
 {
+	int save_errno = errno;
+
 	ereport(LOG,
 			(errmsg("child process received shutdown request signal %d", sig)));
 
@@ -955,6 +963,8 @@ static RETSIGTYPE die(int sig)
 
 			break;
 	}
+
+	errno = save_errno;
 }
 
 /*
@@ -966,6 +976,7 @@ static RETSIGTYPE close_idle_connection(int sig)
 	int i, j;
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	ConnectionInfo *info;
+	int save_errno = errno;
 
 	ereport(DEBUG1,
 			(errmsg("close connection request received")));
@@ -1006,6 +1017,8 @@ static RETSIGTYPE close_idle_connection(int sig)
 			memset(p->info, 0, sizeof(ConnectionInfo));
 		}
 	}
+
+	errno = save_errno;
 }
 
 /*
